/*
 * ChangeApplication.cpp
 * 
 * Written by Matt Pauker
 */

#ifndef CHANGE_APPLICATION_H
#include "ChangeApplication.h"
#endif

#ifndef INFO_WINDOW_H
#include "InfoWindow.h"
#endif

#ifndef CHANGE_CONSTANTS_H
#include "ChangeConstants.h"
#endif

#ifndef CHANGE_UTILS_H
#include "ChangeUtils.h"
#endif

#ifndef _FILE_H
#include <File.h>
#endif


B_DEFINE_CLASS_INFO(ChangeApplication, BApplication);

ChangeApplication::ChangeApplication() : BApplication('CHNG')
{
	// Set up main menu bar
	
	BPopUpMenu *mainMenu;
	BMenuItem *menuItem;
	
	mainMenu = new BPopUpMenu("MainMenu", FALSE, FALSE, B_ITEMS_IN_COLUMN);
	
	menuItem = new BMenuItem("About ChangeType...", new BMessage(B_ABOUT_REQUESTED),
														NULL, NULL);
	menuItem->SetTarget(this);
	mainMenu->AddItem(menuItem);
	
	mainMenu->AddSeparatorItem();
	
	menuItem = new BMenuItem("Open...", new BMessage(OPEN_FILE_PANEL),
														'O', NULL);
	menuItem->SetTarget(this);
	mainMenu->AddItem(menuItem);
	
	mainMenu->AddSeparatorItem();
	
	menuItem = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED),
														'Q', NULL);
	menuItem->SetTarget(this);
	mainMenu->AddItem(menuItem);
	
	SetMainMenu(mainMenu);
	
	// Invalidate the file panel frame and directory so they won't be used the first time
	filePanelFrame = NULL;
	filePanelDir   = NULL;
	
	// Initialize coordinates for first window (Actually, they are both 20 less, since they're
	// incremented before window creation)
	lastX = 60;
	lastY = 40;

	// Initialize the list of windows	
	windowList = new LockableList();
}

ChangeApplication::~ChangeApplication()
{
	delete windowList;
	delete filePanelFrame;
	delete filePanelDir;
}


void ChangeApplication::ReadyToRun()
{
	// If there aren't any windows open when we start (i.e. the user didn't start the app using
	// drag-and-drop), open a file panel so the user can select a file
	if (CountWindows() < 1)
		ShowFilePanel(NULL, NULL, new BMessage(B_REFS_RECEIVED));
}

void ChangeApplication::MessageReceived(BMessage *message)
{
	InfoWindow *theWindow;
	BMessage *msg;
	
	switch (message->what)
	{
		case OPEN_FILE_PANEL:
			ShowFilePanel(NULL, NULL, new BMessage(B_REFS_RECEIVED));
		break;
		case TYPE_CHOSEN:		// User selected a file after clicking 'Choose' button
		case APP_CHOSEN:
			CloseFilePanel();
			
			// Pass the refs along to the window that initiated the file panel
			
			theWindow = (InfoWindow *)message->FindObject("window");
			
			msg = new BMessage(message->what);
			msg->AddRef("refs", message->FindRef("refs"));
			
			theWindow->PostMessage(msg, theWindow);
		break;
		default:
			inherited::MessageReceived(message);
		break;
	}
}

void ChangeApplication::RefsReceived(BMessage *message)
{
	ulong type;
	long numRefs, x;
	bool makeBatch;
	bool controlDown, shiftDown, openedWindow;
	InfoWindow *newWindow;
	screen_info screenInfo;
	record_ref ref;
			
	get_screen_info(&screenInfo);
	
	controlDown = IsKeyDown(B_CONTROL_KEY);
	shiftDown = IsKeyDown(B_SHIFT_KEY);
	openedWindow = FALSE;
	newWindow = NULL;
	
	// Get the refs and open a new window for each file
	message->GetInfo("refs", &type, &numRefs);
	for (x = 0; x < numRefs; x++)
	{
		
		ref = message->FindRef("refs", x);
		if (ref.database >= 0 && ref.record >= 0)
			if (does_ref_conform(ref, "File"))
			{
				// If the file isn't already open in a window, create a new one
				// Otherwise, CheckFileOpen() will bring it to the front

				if (!CheckFileOpen(ref))
				{
					if ((!controlDown) || (!openedWindow))
					// If we need to create a new window...
					{
						// Create a new window and place it on the screen, in a staggered position
						
						lastX += 20;
						lastY += 20;
						
						// Make sure we're not off the screen
						if ((lastX + 190 > screenInfo.frame.right) || (lastY + 85 > screenInfo.frame.bottom))
						{
							lastX = 80;
							lastY = 60;
						}
						
						// Create the new window and show it
						newWindow = new InfoWindow(lastX, lastY);
						newWindow->AddRef(ref);
						
						if (!controlDown)		// Don't want to show it yet if the control key was
							newWindow->Show();	// down; might be more refs to add first
						
						// Add the window to the window list - Lock the list so nothing happens to it while we're
						// modifying it
						windowList->Lock();
						windowList->AddItem(newWindow);
						windowList->Unlock();
						
						openedWindow = TRUE;
				}
				else if (controlDown)
				{
					// The user held down the control key (For a Batch Change) and
					// a window has already been opened, so just add the ref to the window
					newWindow->AddRef(ref);
					newWindow->ResetSigs();
				}
			}
		}
		else if (does_ref_conform(ref, "Folder"))
		// The user dropped a folder onto the app
		{
			if ((!controlDown) || (!openedWindow))
			// If we need to create a new window...
			{
				// Create a new window and place it on the screen, in a staggered position
				
				lastX += 20;
				lastY += 20;
					
				// Make sure we're not off the screen
				if ((lastX + 190 > screenInfo.frame.right) || (lastY + 85 > screenInfo.frame.bottom))
				{
					lastX = 80;
					lastY = 60;
				}
				
				// Create the new window
				newWindow = new InfoWindow(lastX, lastY);
				
				// Add the refs, recursively if the shift key was down
				newWindow->AddRefsInDir(ref, shiftDown);
				newWindow->ResetSigs();

				if (!controlDown)		// Don't want to show it yet if the control key was
					newWindow->Show();	// down; might be more refs to add first

				
				// Add the window to the window list - Lock the list so nothing happens to it while we're
				// modifying it
				windowList->Lock();
				windowList->AddItem(newWindow);
				windowList->Unlock();
				
				openedWindow = TRUE;
			}
			else if (controlDown)
			{
				// The user held down the control key (For a Batch Change) and
				// a window has already been opened, so just add the refs to the window
				newWindow->AddRefsInDir(ref, shiftDown);
				newWindow->ResetSigs();
			}
		}
	}
	if ((newWindow != NULL) && (controlDown))		// Now we can show it
		newWindow->Show();
}

void ChangeApplication::FilePanelClosed(BMessage *message)
{
	bool canceled;
	
	canceled = message->FindBool("canceled");
	
	// We need to quit if the user clicked Cancel and there are no windows open.
	// This would only happen if the user opens the app by double-clicking.
	if (canceled)
		if (CountWindows() < 1)
			PostMessage(B_QUIT_REQUESTED);
			
	// Save directory and frame information
	
	if (filePanelDir == NULL)
		filePanelDir = new record_ref;
	*filePanelDir = message->FindRef("directory");
	
	if (filePanelFrame == NULL)
		filePanelFrame = new BRect;
	*filePanelFrame = message->FindRect("frame");
}

void ChangeApplication::ShowFilePanel(const char *windowTitle, const char *buttonTitle, BMessage *msg)
{
	if (filePanelDir != NULL)
	{
		// Set the frame and current directory to what they were the last time the file panel was run
		
		msg->AddRect("frame", *filePanelFrame);
		msg->AddRef("directory", *filePanelDir);
	}	
	RunFilePanel(windowTitle, buttonTitle, NULL, FALSE, msg);
}

void ChangeApplication::AboutRequested()
{
	BAlert *aboutAlert;
	
	// Display the about box
	aboutAlert = new BAlert("", "ChangeType - Version 1.1\n\nBy Matt Pauker\n\May 17, 1996",
													"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);
	aboutAlert->SetShortcut(0, B_ENTER);
	
	aboutAlert->Go();
}

void ChangeApplication::WindowClosed(BWindow *theWindow)
// Called by a window to tell the application it's closing
{
	// Remove theWindow from the window list
	windowList->Lock();
	windowList->RemoveItem(theWindow);
	windowList->Unlock();
}

bool ChangeApplication::CheckFileOpen(record_ref ref)
// Returns whether or not the file specified by ref is open in a window
// If it is, it activates that window
{
	bool inList;
	int	y, numItems;
	InfoWindow *currentWindow;
	record_ref windowRef;
	
	inList = FALSE;
	currentWindow = NULL;
	
	windowList->Lock();		// We don't want the list changing while we're looking at it
	numItems = windowList->CountItems();
	y = 0;
	
	while ((y < numItems) && (!inList))
	{
		currentWindow = (InfoWindow *)windowList->ItemAt(y);
		if (currentWindow->HasRef(ref))
			inList = TRUE;
		y++;
	}
	windowList->Unlock();
	if (inList)
	{
		currentWindow->Activate();
		currentWindow->Mark(ref);
	}
	return inList;
}
